@@ -28,7 +28,7 @@ from functools import lru_cache
 from typing import TYPE_CHECKING, Sequence
 
 import pyglet
-from pyglet.canvas.xlib import XlibCanvas
+from pyglet.display.xlib import XlibCanvas, XlibScreenXinerama
 from pyglet.event import EventDispatcher
 from pyglet.libs.x11 import cursorfont, xlib
 from pyglet.util import asbytes
@@ -47,8 +47,7 @@ from pyglet.window import (
 
 if TYPE_CHECKING:
     from _ctypes import _Pointer
-
-    from pyglet.gl.xlib import XlibCanvasConfig
+    from pyglet.gl.xlib import XlibDisplayConfig
 
 try:
     from pyglet.libs.x11 import xsync
@@ -125,7 +124,7 @@ ViewEventHandler = _ViewEventHandler
 
 
 class XlibWindow(BaseWindow):
-    config: XlibCanvasConfig
+    config: XlibDisplayConfig
     _x_display: xlib.Display | None = None  # X display connection
     _x_screen_id: int | None = None  # X screen index
     _x_ic: xlib.XIC | None = None  # X input context
@@ -230,8 +229,6 @@ class XlibWindow(BaseWindow):
             root = xlib.XRootWindow(self._x_display, self._x_screen_id)
 
             visual_info = self.config.get_visual_info()
-            if self.style in ('transparent', 'overlay'):
-                xlib.XMatchVisualInfo(self._x_display, self._x_screen_id, 32, xlib.TrueColor, visual_info)
 
             visual = visual_info.visual
             visual_id = xlib.XVisualIDFromVisual(visual)
@@ -252,6 +249,9 @@ class XlibWindow(BaseWindow):
             #            unconditionally.
             mask = xlib.CWColormap | xlib.CWBitGravity | xlib.CWBackPixel
 
+            self._dpi = self._screen.get_dpi()
+            self._scale = self._screen.get_scale() if pyglet.options.dpi_scaling == "stretch" else 1.0
+
             if self.style in ('transparent', 'overlay'):
                 mask |= xlib.CWBorderPixel
                 window_attributes.border_pixel = 0
@@ -264,6 +264,10 @@ class XlibWindow(BaseWindow):
             else:
                 width, height = self._width, self._height
                 self._view_x = self._view_y = 0
+                if pyglet.options.dpi_scaling in ("scaled", "stretch"):
+                    w, h = self.get_requested_size()
+                    self._width = width = int(w * self.scale)
+                    self._height = height = int(h * self.scale)
 
             self._window = xlib.XCreateWindow(self._x_display, root,
                                               0, 0, width, height, 0, visual_info.depth,
@@ -285,20 +289,10 @@ class XlibWindow(BaseWindow):
             self.context.attach(self.canvas)
             self.context.set_vsync(self._vsync)  # XXX ?
 
-            # Setting null background pixmap disables drawing the background,
-            # preventing flicker while resizing (in theory).
-            #
-            # Issue 287: Compiz on Intel/Mesa doesn't draw window decoration if
-            #            this is called.  As it doesn't seem to have any
-            #            effect anyway, it's just commented out.
-            # xlib.XSetWindowBackgroundPixmap(self._x_display, self._window, 0)
-
-            self._enable_xsync = (pyglet.options['xsync'] and
-                                  self.display._enable_xsync and  # noqa: SLF001
-                                  self.config.double_buffer)
+            self._enable_xsync = self.display._enable_xsync and self.config.double_buffer
 
             # Set supported protocols
-            protocols = []
+            protocols = list()
             protocols.append(xlib.XInternAtom(self._x_display, asbytes('WM_DELETE_WINDOW'), False))
             if self._enable_xsync:
                 protocols.append(xlib.XInternAtom(self._x_display,
@@ -307,6 +301,11 @@ class XlibWindow(BaseWindow):
             protocols = (c_ulong * len(protocols))(*protocols)
             xlib.XSetWMProtocols(self._x_display, self._window, protocols, len(protocols))
 
+            # Overlay should allow mouse to pass through and stay on top.
+            if self._style == "overlay":
+                self.set_mouse_passthrough(True)
+                self._set_wm_state("_NET_WM_STATE_ABOVE")
+
             # Create window resize sync counter
             if self._enable_xsync:
                 value = xsync.XSyncValue()
@@ -337,8 +336,8 @@ class XlibWindow(BaseWindow):
                 self._xdnd_format = None
                 self._xdnd_position = (0, 0)  # For position callback.
 
-                VERSION = c_ulong(int(XDND_VERSION))
-                ptr = pointer(VERSION)
+                _version = c_ulong(int(XDND_VERSION))
+                ptr = pointer(_version)
 
                 xlib.XChangeProperty(self._x_display, self._window,
                                      self._xdnd_atoms['XdndAware'], XA_ATOM, 32,
@@ -349,16 +348,8 @@ class XlibWindow(BaseWindow):
         attributes = xlib.XSetWindowAttributes()
         attributes_mask = 0
 
-        self._override_redirect = False
         if self._fullscreen:
-            if pyglet.options['xlib_fullscreen_override_redirect']:
-                # Try not to use this any more, it causes problems; disabled
-                # by default in favour of _NET_WM_STATE_FULLSCREEN.
-                attributes.override_redirect = self._fullscreen
-                attributes_mask |= xlib.CWOverrideRedirect
-                self._override_redirect = True
-            else:
-                self._set_wm_state('_NET_WM_STATE_FULLSCREEN')
+            self._set_wm_state('_NET_WM_STATE_FULLSCREEN')
 
         if self._fullscreen:
             xlib.XMoveResizeWindow(self._x_display, self._window,
@@ -448,6 +439,16 @@ class XlibWindow(BaseWindow):
         self._applied_mouse_exclusive = None
         self._update_exclusivity()
 
+    def set_mouse_passthrough(self, state: bool) -> None:
+        """Sets the clickable area in the application to an empty region if enabled."""
+        if state:
+            region = xlib.XCreateRegion()
+            xsync.XShapeCombineRegion(self._x_display, self._window, xsync.ShapeInput, 0, 0, region, xsync.ShapeSet)
+            xlib.XDestroyRegion(region)
+        else:
+            # Reset input shape to default
+            xsync.XShapeCombineMask(self._x_display, self._window, xsync.ShapeInput, 0, 0, 0, xsync.ShapeSet)
+
     def _map(self) -> None:
         if self._mapped:
             return
@@ -466,13 +467,9 @@ class XlibWindow(BaseWindow):
         xlib.XSelectInput(self._x_display, self._window, self._default_event_mask)
         self._mapped = True
 
-        if self._override_redirect:
-            # Possibly an override_redirect issue.
-            self.activate()
-
         self._update_view_size()
 
-        self.dispatch_event('on_resize', self._width, self._height)
+        self.dispatch_event('_on_internal_resize', self._width, self._height)
         self.dispatch_event('on_show')
         self.dispatch_event('on_expose')
 
@@ -582,7 +579,7 @@ class XlibWindow(BaseWindow):
             self.set_maximum_size(width, height)
         xlib.XResizeWindow(self._x_display, self._window, width, height)
         self._update_view_size()
-        self.dispatch_event('on_resize', width, height)
+        self.dispatch_event('_on_internal_resize', width, height)
 
     def _update_view_size(self) -> None:
         xlib.XResizeWindow(self._x_display, self._view, self._width, self._height)
@@ -613,6 +610,15 @@ class XlibWindow(BaseWindow):
                                    byref(child))
         return x.value, y.value
 
+    def get_framebuffer_size(self) -> tuple[int, int]:
+        return self._width, self._height
+
+    def get_size(self) -> tuple[int, int]:
+        if pyglet.options.dpi_scaling == "stretch":
+            return int(self._width / self.scale), int(self._height / self.scale)
+
+        return self._width, self._height
+
     def activate(self) -> None:
         # Issue 218
         if self._x_display and self._window:
@@ -737,7 +743,10 @@ class XlibWindow(BaseWindow):
                 y = self._height // 2
                 self._mouse_exclusive_client = x, y
                 self.set_mouse_position(x, y)
-            elif self._fullscreen and not self.screen._xinerama:  # noqa: SLF001
+            elif self._fullscreen:  # noqa: SLF001
+                if isinstance(self.screen, XlibScreenXinerama) and self.screen._xinerama:
+                    return
+
                 # Restrict to fullscreen area (prevent viewport scrolling)
                 self.set_mouse_position(0, 0)
                 r = xlib.XGrabPointer(self._x_display, self._view,
@@ -906,8 +915,6 @@ class XlibWindow(BaseWindow):
 
             text = text_bytes.decode('utf-8')
 
-        self._clipboard_str = text
-
         xlib.XFree(data)
         return text
 
@@ -947,7 +954,7 @@ class XlibWindow(BaseWindow):
                 raise XlibException(msg)
         xlib.XSetTextProperty(self._x_display, self._window, byref(text_property), atom)
         # XXX <rj> Xlib doesn't like us freeing this
-        # xlib.XFree(text_property.value)  # noqa: ERA001
+        # xlib.XFree(text_property.value)
 
     def _set_atoms_property(self, name: str, values: list[str], mode: int = xlib.PropModeReplace) -> None:
         name_atom = xlib.XInternAtom(self._x_display, asbytes(name), False)
@@ -1237,8 +1244,8 @@ class XlibWindow(BaseWindow):
     @ViewEventHandler
     @XlibEventHandler(xlib.MotionNotify)
     def _event_motionnotify_view(self, ev: xlib.XEvent) -> None:
-        x = ev.xmotion.x
-        y = self.height - ev.xmotion.y - 1
+        x = ev.xmotion.x / self._scale
+        y = self.height - 1 - ev.xmotion.y / self._scale
 
         if self._mouse_in_window:
             dx = x - self._mouse_x
@@ -1298,8 +1305,8 @@ class XlibWindow(BaseWindow):
 
         if buttons:
             # Drag event
-            x = ev.xmotion.x - self._view_x
-            y = self._height - (ev.xmotion.y - self._view_y - 1)
+            x = (ev.xmotion.x - self._view_x) / self._scale
+            y = self._height - 1 - self._view_y - ev.xmotion.y / self._scale
 
             if self._mouse_in_window:
                 dx = x - self._mouse_x
@@ -1526,7 +1533,7 @@ class XlibWindow(BaseWindow):
         """Translate mouse button values to match mouse constants.
 
         Given a Xevent.xbutton.button value, convert it to the mouse
-        contants defined in :py:module:`~pyglet.window.mouse`. This
+        contents defined in :py:module:`~pyglet.window.mouse`. This
         means shifting the value, and also skipping over values of
         4~7, which are used for boolean scrolling.
         """
@@ -1539,8 +1546,8 @@ class XlibWindow(BaseWindow):
     @XlibEventHandler(xlib.ButtonPress)
     @XlibEventHandler(xlib.ButtonRelease)
     def _event_button(self, ev: xlib.XEvent) -> None:
-        x = ev.xbutton.x
-        y = self.height - ev.xbutton.y
+        x = ev.xbutton.x / self._scale
+        y = self.height - 1 - ev.xbutton.y / self._scale
 
         modifiers = self._translate_modifiers(ev.xbutton.state)
 
@@ -1579,8 +1586,8 @@ class XlibWindow(BaseWindow):
     @XlibEventHandler(xlib.EnterNotify)
     def _event_enternotify(self, ev: xlib.XEvent) -> None:
         # mouse position
-        x = self._mouse_x = ev.xcrossing.x
-        y = self._mouse_y = self.height - ev.xcrossing.y
+        x = self._mouse_x = ev.xcrossing.x / self._scale
+        y = self._mouse_y = self.height - 1 - ev.xcrossing.y / self._scale
         self._mouse_in_window = True
 
         # XXX there may be more we could do here
@@ -1589,8 +1596,8 @@ class XlibWindow(BaseWindow):
     @ViewEventHandler
     @XlibEventHandler(xlib.LeaveNotify)
     def _event_leavenotify(self, ev: xlib.XEvent) -> None:
-        x = self._mouse_x = ev.xcrossing.x
-        y = self._mouse_y = self.height - ev.xcrossing.y
+        x = self._mouse_x = ev.xcrossing.x / self._scale
+        y = self._mouse_y = self.height - 1 - ev.xcrossing.y / self._scale
         self._mouse_in_window = False
         self.dispatch_event('on_mouse_leave', x, y)
 
@@ -1607,10 +1614,10 @@ class XlibWindow(BaseWindow):
         w, h = ev.xconfigure.width, ev.xconfigure.height
         x, y = ev.xconfigure.x, ev.xconfigure.y
         if self._width != w or self._height != h:
-            self._width = w
-            self._height = h
+            self._width = max(1, w)
+            self._height = max(1, h)
             self._update_view_size()
-            self.dispatch_event('on_resize', self._width, self._height)
+            self.dispatch_event('_on_internal_resize', self._width, self._height)
         if self._x != x or self._y != y:
             self.dispatch_event('on_move', x, y)
             self._x = x
@@ -1697,7 +1704,7 @@ class XlibWindow(BaseWindow):
         xlib.XSendEvent(self._x_display, request.requestor, 0, 0, byref(out_event))
 
         # Seems to work find without it. May add later.
-        # xlib.XSync(self._x_display, False)  # noqa: ERA001
+        # xlib.XSync(self._x_display, False)
 
 
 __all__ = ['XlibEventHandler', 'XlibWindow']
